
/* Chrysalide - Outil d'analyse de fichiers binaires
 * loadedpanel.c - équivalent Python du fichier "glibext/gloadedpanel.h"
 *
 * Copyright (C) 2018-2019 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "loadedpanel.h"


#include <assert.h>
#include <pygobject.h>


#include <i18n.h>
#include <glibext/gloadedpanel.h>


#include "linecursor.h"
#include "../access.h"
#include "../helpers.h"



/* S'assure qu'un emplacement donné est visible à l'écran. */
static PyObject *py_loaded_panel_scroll_to_cursor(PyObject *, PyObject *);

/* Définit les constantes pour l'affichage des contenus chargés. */
static bool py_loaded_panel_define_constants(PyTypeObject *);



/******************************************************************************
*                                                                             *
*  Paramètres  : self = classe représentant un tampon de code.                *
*                args = arguments fournis à l'appel.                          *
*                                                                             *
*  Description : S'assure qu'un emplacement donné est visible à l'écran.      *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_loaded_panel_scroll_to_cursor(PyObject *self, PyObject *args)
{
    GLineCursor *cursor;                    /* Emplacement à cibler        */
    unsigned long tweak;                    /* Adapation à effectuer       */
    int move;                               /* Déplacement à l'écran ?     */
    int ret;                                /* Bilan de lecture des args.  */
    GLoadedPanel *panel;                    /* Panneau à manipuler         */

    ret = PyArg_ParseTuple(args, "O&kp", convert_to_line_cursor, &cursor, &tweak, &move);
    if (!ret) return NULL;

    if (!IS_VALID_STP(tweak))
    {
        PyErr_SetString(PyExc_ValueError, _("invalid position tweak"));
        return NULL;
    }

    panel = G_LOADED_PANEL(pygobject_get(self));

    g_loaded_panel_scroll_to_cursor(panel, cursor, tweak, move);

    Py_RETURN_NONE;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : obj_type = type dont le dictionnaire est à compléter.        *
*                                                                             *
*  Description : Définit les constantes pour l'affichage des contenus chargés.*
*                                                                             *
*  Retour      : true en cas de succès de l'opération, false sinon.           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static bool py_loaded_panel_define_constants(PyTypeObject *obj_type)
{
    bool result;                            /* Bilan à retourner           */

    result = true;

    result &= PyDict_AddULongMacro(obj_type, SPT_RAW);
    result &= PyDict_AddULongMacro(obj_type, SPT_TOP);
    result &= PyDict_AddULongMacro(obj_type, SPT_CENTER);
    result &= PyDict_AddULongMacro(obj_type, SPT_BOTTOM);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Fournit un accès à une définition de type à diffuser.        *
*                                                                             *
*  Retour      : Définition d'objet pour Python.                              *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

PyTypeObject *get_python_loaded_panel_type(void)
{
    static PyMethodDef py_loaded_panel_methods[] = {
        {
            "scroll_to_cursor", py_loaded_panel_scroll_to_cursor,
            METH_VARARGS,
            "scroll_to_cursor($self, cursor, tweak, move, /)\n--\n\nEnsure a given address is displayed in the view panel."
        },
        { NULL }
    };

    static PyGetSetDef py_loaded_panel_getseters[] = {
        { NULL }
    };

    static PyTypeObject py_loaded_panel_type = {

        PyVarObject_HEAD_INIT(NULL, 0)

        .tp_name        = "pychrysalide.glibext.LoadedPanel",
        //.tp_basicsize   = sizeof(PyGObject),

        .tp_flags       = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,

        .tp_doc         = "PyChrysalide loaded panel",

        .tp_methods     = py_loaded_panel_methods,
        .tp_getset      = py_loaded_panel_getseters,

    };

    return &py_loaded_panel_type;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : module = module dont la définition est à compléter.          *
*                                                                             *
*  Description : Prend en charge l'objet 'pychrysalide.glibext.LoadedPanel'.  *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool ensure_python_loaded_panel_is_registered(void)
{
    PyTypeObject *type;                     /* Type Python 'LineGenerator' */
    PyObject *module;                       /* Module à recompléter        */
    PyObject *dict;                         /* Dictionnaire du module      */

    type = get_python_loaded_panel_type();

    if (!PyType_HasFeature(type, Py_TPFLAGS_READY))
    {
        module = get_access_to_python_module("pychrysalide.glibext");

        dict = PyModule_GetDict(module);

        if (!register_interface_for_pygobject(dict, G_TYPE_LOADED_PANEL, type))
            return false;

        if (!py_loaded_panel_define_constants(type))
            return false;

    }

    return true;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : arg = argument quelconque à tenter de convertir.             *
*                dst = destination des valeurs récupérées en cas de succès.   *
*                                                                             *
*  Description : Tente de convertir en panneau de contenu chargé.             *
*                                                                             *
*  Retour      : Bilan de l'opération, voire indications supplémentaires.     *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

int convert_to_loaded_panel(PyObject *arg, void *dst)
{
    int result;                             /* Bilan à retourner           */

    result = PyObject_IsInstance(arg, (PyObject *)get_python_loaded_panel_type());

    switch (result)
    {
        case -1:
            /* L'exception est déjà fixée par Python */
            result = 0;
            break;

        case 0:
            PyErr_SetString(PyExc_TypeError, "unable to convert the provided argument to loaded panel");
            break;

        case 1:
            *((GLoadedPanel **)dst) = G_LOADED_PANEL(pygobject_get(arg));
            break;

        default:
            assert(false);
            break;

    }

    return result;

}
